@@ -2,7 +2,7 @@
 #ifndef _GLIBMM_USTRING_H
 #define _GLIBMM_USTRING_H
 
-/* $Id: ustring.h,v 1.6 2004/05/26 20:33:03 daniel Exp $ */
+/* $Id: ustring.h,v 1.15 2006/09/19 20:48:15 murrayc Exp $ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
@@ -232,65 +232,71 @@ public:
 
 #endif /* GLIBMM_HAVE_SUN_REVERSE_ITERATOR */
 
-  static const size_type npos = std::string::npos;
+#ifdef GLIBMM_HAVE_ALLOWS_STATIC_INLINE_NPOS
+  static GLIBMM_API const size_type npos = std::string::npos;
+#else
+  //The IRIX MipsPro compiler says "The indicated constant value is not known",
+  //so we need to initalize the static member data elsewhere.
+  static GLIBMM_API const size_type npos;
+#endif
 
-  /** Default constructor, which creates an empty string.
+  /*! Default constructor, which creates an empty string.
    */
   ustring();
   
   ~ustring();
 
-  /** Construct a ustring as a copy of a ustring
+  /*! Construct a ustring as a copy of another ustring.
    * @param other A source string.
    */
   ustring(const ustring& other);
 
-  /** Assign the value of a string to this string.
-   * @param other A source string
+  /*! Assign the value of another string to this string.
+   * @param other A source string.
    */ 
   ustring& operator=(const ustring& other);
 
-  /** Swap contents with another string.
+  /*! Swap contents with another string.
    * @param other String to swap with.
    */
   void swap(ustring& other);
 
-  /** Construct a ustring as a copy of a std::string
-   * @param src A source std::string containing text encoded as UTF-8.
+  /*! Construct a ustring as a copy of another std::string.
+   * @param src A source <tt>std::string</tt> containing text encoded as UTF-8.
    */
   ustring(const std::string& src);
 
-  /** Construct a ustring as a copy of a substring
-   * @param src Source Glib::ustring.
+  /*! Construct a ustring as a copy of a substring.
+   * @param src %Source ustring.
    * @param i Index of first character to copy from.
-   * @param n Number of characters to copy (default to copying the remainder).
+   * @param n Number of UTF-8 characters to copy (defaults to copying the remainder).
    */
   ustring(const ustring& src, size_type i, size_type n=npos);
 
-  /** Construct a ustring as a copy of a substring
-   * @param src Source C string encoded as UTF-8.
-   * @param n Number of characters to copy.
+  /*! Construct a ustring as a partial copy of a C string.
+   * @param src %Source C string encoded as UTF-8.
+   * @param n Number of UTF-8 characters to copy.
    */
   ustring(const char* src, size_type n);
 
-  /** Construct a ustring as a copy of a substring
-   * @param src Source C string encoded as UTF-8.
+  /*! Construct a ustring as a copy of a C string.
+   * @param src %Source C string encoded as UTF-8.
    */
   ustring(const char* src);
 
-  /** Construct a  ustring as multiple characters.
+  /*! Construct a ustring as multiple characters.
    * @param n Number of characters.
-   * @param uc UTF-8 character to use.
+   * @param uc UCS-4 code point to use.
    */  
   ustring(size_type n, gunichar uc);
 
-  /** Construct a ustring as multiple characters.
+  /*! Construct a ustring as multiple characters.
    * @param n Number of characters.
    * @param c ASCII character to use.
    */  
   ustring(size_type n, char c);
 
-  /** Construct a ustring as a copy of a range.
+  /*! Construct a ustring as a copy of a range.
    * @param pbegin Start of range.
    * @param pend End of range.
    */  
@@ -481,9 +487,29 @@ public:
 //! @name Retrieve the string's size.
 //! @{
 
-  bool      empty()  const;
+  /** Returns true if the string is empty. Equivalent to *this == "".
+   * @result Whether the string is empty.
+   */
+  bool empty()  const;
+
+  /** Returns the number of characters in the string, not including any null-termination.
+   * @result The number of UTF-8 characters.
+   *
+   * @see bytes(), empty() 
+   */
   size_type size()   const;
+
+  //We have length() as well as size(), because std::string has both.
+
+  /** This is the same as size().
+   */
   size_type length() const;
+  
+  /** Returns the number of bytes in the string, not including any null-termination.
+   * @result The number of bytes.
+   *
+   * @see size(), empty()
+   */
   size_type bytes()  const;
 
 //! @}
@@ -544,11 +570,11 @@ public:
 //! @{
 
   /*! Returns a new UTF-8 string with all characters characters converted to
-   * their lowercase equivalent, while honoring the current locale.  The
+   * their uppercase equivalent, while honoring the current locale.  The
    * resulting string may change in the number of bytes as well as in the
    * number of characters.  For instance, the German sharp&nbsp;s
-   * <tt>"&szlig;"</tt> will be replaced by two characters <tt>"SS"</tt>
-   * because there is no capital <tt>"&szlig;"</tt>.
+   * <tt>&quot;&szlig;&quot;</tt> will be replaced by two characters
+   * <tt>"SS"</tt> because there is no capital <tt>&quot;&szlig;&quot;</tt>.
    */
   ustring uppercase() const;
 
@@ -578,7 +604,23 @@ private:
   template <class In, class ValueType = typename Glib::IteratorTraits<In>::value_type>
 #endif
   struct SequenceToString;
+  
+  //The Tru64 compiler needs these partial specializations to be declared here,
+  //as well as defined later. That's probably correct. murrayc.
+  template <class In>
+  struct SequenceToString<In, char>;
 
+  template <class In>
+  struct SequenceToString<In, gunichar>;
+  
+  /*
+  template <>
+  struct ustring::SequenceToString<Glib::ustring::iterator, gunichar>;
+
+  template <>
+  struct ustring::SequenceToString<Glib::ustring::const_iterator, gunichar>;
+  */
+  
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
   std::string string_;
@@ -592,25 +634,25 @@ struct ustring::SequenceToString
 {};
 
 template <class In>
-struct ustring::SequenceToString<In,char> : public std::string
+struct ustring::SequenceToString<In, char> : public std::string
 {
   SequenceToString(In pbegin, In pend);
 };
 
 template <class In>
-struct ustring::SequenceToString<In,gunichar> : public std::string
+struct ustring::SequenceToString<In, gunichar> : public std::string
 {
   SequenceToString(In pbegin, In pend);
 };
 
 template <>
-struct ustring::SequenceToString<Glib::ustring::iterator,gunichar> : public std::string
+struct ustring::SequenceToString<Glib::ustring::iterator, gunichar> : public std::string
 {
   SequenceToString(Glib::ustring::iterator pbegin, Glib::ustring::iterator pend);
 };
 
 template <>
-struct ustring::SequenceToString<Glib::ustring::const_iterator,gunichar> : public std::string
+struct ustring::SequenceToString<Glib::ustring::const_iterator, gunichar> : public std::string
 {
   SequenceToString(Glib::ustring::const_iterator pbegin, Glib::ustring::const_iterator pend);
 };
